import os
import tempfile

from damona.builders import BuilderFromDocker, BuilderFromSingularityRecipe
from . import test_dir

# reach too many requests ??
def test_docker_alpine(monkeypatch):

    # explcitily named the output file
    with tempfile.TemporaryDirectory() as td:
        bb = BuilderFromDocker()
        bb.build("alpine", destination=td + "/alpine.img")

        # we start again to test the force option
        bb.build("alpine", destination=td + "/alpine.img", force=True)

        # we start again to test the input set to yes
        monkeypatch.setattr("builtins.input", lambda _: "yes")
        bb.build("alpine", destination=td + "/alpine.img")

        # we start again to test the input set to no
        monkeypatch.setattr("builtins.input", lambda _: "no")
        bb.build("alpine", destination=td + "/alpine.img")

        # we start again to test the input set to unexpected value
        monkeypatch.setattr("builtins.input", lambda _: "dummy")
        try:
            bb.build("alpine", destination=td + "/alpine.img")
            assert False
        except SystemExit:
            assert True

    try:
        with tempfile.TemporaryDirectory() as td:
            bb = BuilderFromDocker()
            bb.build("alpine", destination=td + "/alpine.wrong_extension")
        assert False
    except:
        assert True


def test_singularity_recipe(monkeypatch):
    bb = BuilderFromSingularityRecipe()

    try:
        bb.build(f"{test_dir}/data/dummy")
        assert False
    except SystemExit:
        assert True

    with tempfile.TemporaryDirectory() as td:
        bb.build(f"{test_dir}/data/Singularity.testing_1.0.0", destination=td + "/test.img")

        # we start again to test the force option
        bb.build(f"{test_dir}/data/Singularity.testing_1.0.0", destination=td + "/test.img", force=True)

        # we start again to test the input set to yes
        monkeypatch.setattr("builtins.input", lambda _: "yes")
        bb.build(f"{test_dir}/data/Singularity.testing_1.0.0", destination=td + "/test.img")

        # we start again to test the input set to no
        monkeypatch.setattr("builtins.input", lambda _: "no")
        bb.build(f"{test_dir}/data/Singularity.testing_1.0.0", destination=td + "/test.img")

        # we start again to test the input set to unexpected value
        monkeypatch.setattr("builtins.input", lambda _: "dummy")
        try:
            bb.build(f"{test_dir}/data/Singularity.testing_1.0.0", destination=td + "/test.img")
            assert False
        except SystemExit:
            assert True



import damona
from damona import zenodo

from . import test_dir


deposit = {
    "conceptdoi": "10.5072/zenodo.960007",
    "conceptrecid": "960007",
    "created": "2021-11-13T20:15:42.926938+00:00",
    "doi": "10.5072/zenodo.960008",
    "doi_url": "https://doi.org/10.5072/zenodo.960008",
    "files": [
        {
            "checksum": "4949d8da803d970e74de7ce898ed3c6b",
            "filename": "art_2.5.8.img",
            "filesize": 339525632,
            "id": "de1a2781-7d3f-4322-97d4-52fcd9df29c0",
            "links": {
                "download": "https://sandbox.zenodo.org/api/files/7b677fd2-2722-4f17-a270-3cef204f9858/art_2.5.8.img",
                "self": "https://sandbox.zenodo.org/api/deposit/depositions/960008/files/de1a2781-7d3f-4322-97d4-52fcd9df29c0",
            },
        }
    ],
    "id": 960008,
    "links": {
        "badge": "https://sandbox.zenodo.org/badge/doi/10.5072/zenodo.960008.svg",
        "bucket": "https://sandbox.zenodo.org/api/files/7b677fd2-2722-4f17-a270-3cef204f9858",
        "conceptbadge": "https://sandbox.zenodo.org/badge/doi/10.5072/zenodo.960007.svg",
        "conceptdoi": "https://doi.org/10.5072/zenodo.960007",
        "doi": "https://doi.org/10.5072/zenodo.960008",
        "latest": "https://sandbox.zenodo.org/api/records/960008",
        "latest_html": "https://sandbox.zenodo.org/record/960008",
        "record": "https://sandbox.zenodo.org/api/records/960008",
        "record_html": "https://sandbox.zenodo.org/record/960008",
    },
    "metadata": {
        "access_right": "open",
        "communities": [{"identifier": "zenodo"}],
        "creators": [{"affiliation": "Institut Pasteur", "name": "Cokelaer, Thomas"}],
        "description": 'Singularity image(s) of art software to be used and installed with damona\n(<a href="https://damona.readthedocs.org">See https://damona.readthedocs.org</a>) for reproducible bioinformatics\nanalysis.',
        "doi": "10.5072/zenodo.960008",
        "keywords": ["singularity", "damona", "bioinformatics", "reproducibility", "container"],
        "license": "CC-BY-4.0",
        "prereserve_doi": {"doi": "10.5072/zenodo.960008", "recid": 960008},
        "publication_date": "2021-11-13",
        "title": "Singularity image of art software",
        "upload_type": "physicalobject",
        "version": "v2.5.8",
    },
    "modified": "2021-11-13T20:22:57.340141+00:00",
    "owner": 18543,
    "record_id": 960008,
    "state": "done",
    "submitted": True,
    "title": "Singularity image of art software",
}




def test(mocker):

    mocker.patch("damona.zenodo.Zenodo.create_new_deposition", return_values={})

    try:
        z = zenodo.Zenodo("sandbox.zenodo")
    except:
        SystemExit

    z = zenodo.Zenodo("sandbox.zenodo", token="dummy", affiliation="dummy", author="dummy")
    z.params

    # z._to_registry(deposit)
    #data1 = z.get_metadata("fastqc", "v0.1.1")
    #data2 = z.get_metadata("fastqc", "0.1.1")
    #assert data1 == data2

    z.get_id(deposit)

def test_get_stat():
    from damona.zenodo import get_stats_id
    stats = get_stats_id("5708811")
    assert stats >0



import builtins
import subprocess
from damona import script
from damona import Config
from damona import Damona
from damona import Environ
import pytest
from click.testing import CliRunner
import mock


def test_damon_builder_docker(tmpdir):
    directory = tmpdir.mkdir("images")
    destination = directory / "alpine_1.0.0.img"
    from click.testing import CliRunner

    runner = CliRunner()

    results = runner.invoke(script.build, ["docker://alpine", " --destination", destination])
    # assert results.exit_code == 0


def test_damona_version():
    cmd = "damona --version"
    status = subprocess.call(cmd.split())
    assert status == 0


def test_activate_deactivate(monkeypatch):
    setup()
    manager = Damona()
    monkeypatch.setenv("DAMONA_ENV", str(manager.damona_path / "envs/damona__testing__"))
    runner = CliRunner()
    results = runner.invoke(script.activate, ["damona__testing__"])
    assert results.exit_code == 0
    results = runner.invoke(script.deactivate, ["damona__testing__"])
    assert results.exit_code == 0

    teardown()

def test_damona_clean():
    runner = CliRunner()
    results = runner.invoke(script.clean, [])
    assert results.exit_code == 0

def test_damona_info():
    runner = CliRunner()
    results = runner.invoke(script.info, ["base"])
    assert results.exit_code == 0

    results = runner.invoke(script.info, ["__dummy__yummy__"])
    assert results.exit_code == 1


def test_damona_create_and_export(monkeypatch):
    setup()

    manager = Damona()

    # make sure it exists
    runner = CliRunner()
    results = runner.invoke(script.env, ["--create", "damona__testing__"])
    assert results.exit_code == 0

    monkeypatch.setenv("DAMONA_ENV", str(manager.damona_path / "envs/damona__testing__"))
    cmd = "damona install fastqc"
    status = subprocess.call(cmd.split())
    assert status == 0
    cmd = "damona install fastqc --force"
    status = subprocess.call(cmd.split())
    assert status == 0

    teardown()

def test_env():

    # list the packages
    runner = CliRunner()
    results = runner.invoke(script.env, [])
    assert results.exit_code == 0

    # diskusage
    runner = CliRunner()
    results = runner.invoke(script.env, ["--disk-usage"])
    assert results.exit_code == 0

    # wrong arguments mutually exclusive
    runner = CliRunner()
    results = runner.invoke(script.env, ["--create", "test", "--delete" , "test"])
    assert results.exit_code == 1
    results = runner.invoke(script.env, ["--create", "test", "--rename" , "test"])
    assert results.exit_code == 1
    results = runner.invoke(script.env, ["--rename", "test", "--delete" , "test"])
    assert results.exit_code == 1

    # create damona__testing__ env
    setup()

    # delete it
    import mock
    with mock.patch.object(builtins, "input", lambda _: "y"):
        results = runner.invoke(script.env, ["--delete", "damona__testing__"])
        assert results.exit_code == 0

    # and install again
    results = runner.invoke(script.env, ["--create", "damona__testing__"])
    assert results.exit_code == 0

    with mock.patch.object(builtins, "input", lambda _: ""):
        results = runner.invoke(script.env, ["--rename", "damona__testing__"])
        assert results.exit_code == 1
        results = runner.invoke(script.env, ["--rename", "damona__testing__", 
            "--new-name", "damona__testing__new"])
        assert results.exit_code == 0

    results = runner.invoke(script.env, ["--rename", "damona__testing__new", 
        "--new-name", "damona__testing__", "--force"])
    assert results.exit_code == 0

    teardown()

def test_search():
    runner = CliRunner()
    results = runner.invoke(script.search, [])
    assert results.exit_code == 2

    results = runner.invoke(script.search, ["*"])
    assert results.exit_code == 0

    results = runner.invoke(script.search, ["fastqc"])
    assert results.exit_code == 0

    results = runner.invoke(script.search, ["fastqc", "--binaries-only"])
    assert results.exit_code == 0

    results = runner.invoke(script.search, ["fastqc", "--url", "damona"])
    assert results.exit_code == 0


def test_stats():
    runner = CliRunner()
    results = runner.invoke(script.stats, [])
    assert results.exit_code == 0


def test_export():
    runner = CliRunner()
    setup()

    if "damona__testing__" not in Environ().environment_names:
        results = runner.invoke(script.env, ["--create", "damona__testing__"])
        assert results.exit_code == 0

    results = runner.invoke(script.export, ["damona__testing__"])
    assert results.exit_code == 0

    teardown()

def test_import_bundle(monkeypatch):
    manager = Damona()
    runner = CliRunner()

    setup()

    monkeypatch.setenv("DAMONA_ENV", str(manager.damona_path / "envs/damona__testing__"))
    results = runner.invoke(script.install, ["fastqc"])
    results = runner.invoke(script.export, ["damona__testing__"])
    results = runner.invoke(script.env, ["--create","damona__testing2__", 
        "--from-bundle", "damona_damona__testing__.tar"])
    
    # supress this temporary environment
    with mock.patch.object(builtins, "input", lambda _: "y"):
        results = runner.invoke(script.env, ["--delete", "damona__testing2__"])
        assert results.exit_code == 0

    # supress the damona__testing__ temporary environment
    teardown()

def teardown():
    runner = CliRunner()
    with mock.patch.object(builtins, "input", lambda _: "y"):
        results = runner.invoke(script.env, ["--delete", "damona__testing__"])
        assert results.exit_code == 0



def setup():
    runner = CliRunner()
    if "damona__testing__" not in Environ().environment_names:
        results = runner.invoke(script.env, ["--create", "damona__testing__"])
        assert results.exit_code == 0



def test_install_remove(monkeypatch):
    runner = CliRunner()

    setup()
    manager = Damona()
    monkeypatch.setenv("DAMONA_ENV", str(manager.damona_path / "envs/damona__testing__"))

    results = runner.invoke(script.install, ["fastqc"])
    assert results.exit_code == 0
    results = runner.invoke(script.remove, ["fastqc"])
    assert results.exit_code == 0

    teardown()




from damona.registry import Software, Registry, ImageName
import glob


def test_image_name():

    image = ImageName("fastqc_0.11.8.img")
    assert image.name == "fastqc"
    assert image.version == "0.11.8"

    image = ImageName("fastqc_other_0.11.8.img")
    assert image.name == "fastqc_other"
    assert image.version == "0.11.8"

    try:
        ImageName("fastqc_0.11.img")
    except NameError:
        assert True

    try:
        ImageName("fastqc_0.11.8.sif")
    except NameError:
        assert True

    try:
        ImageName("fastqc0.11.8.img")
    except NameError:
        assert True


def test_single_registry():

    from damona.recipes import __path__

    _registry_files = glob.glob(__path__[0] + "/*/registry.yaml")

    # binaries is not provided and found from the registry name
    salmon_registry = [x for x in _registry_files if "salmon" in x][0]
    sr = Software(salmon_registry)
    assert sr.releases["1.3.0"].binaries == ["salmon"]

    # Sofware can take as input a full path name, a dictionary, a name
    sr = Software("salmon")
    assert sr.releases["1.3.0"].binaries == ["salmon"]
    sr.releases.last_release

    print(sr)
    sr

    # __repr__ test
    print(sr.releases["1.3.0"])
    sr.releases["1.3.0"]

    # binaries is provided explicitly in the general section
    fastqc_registry = [x for x in _registry_files if "/fastqc/" in x][0]
    sr = Software(fastqc_registry)
    assert sr.releases["0.11.9"].binaries == ["fastqc"]

    # binaries are provided explicitly in the releases section. Here there are
    # two releases with different binary names
    kraken_registry = [x for x in _registry_files if "kraken" in x][0]
    sr = Software(kraken_registry)
    assert "kraken" in sr.releases["1.1.0"].binaries
    assert "kraken2" in sr.releases["2.0.9"].binaries

    sr.check()
    sr.binaries
    sr.md5
    sr.releases
    sr.versions

    sr = Software("rtools")
    assert sr.releases["1.0.0"].binaries == ["R", "Rscript"]


def test_registry():

    reg = Registry()
    reg.get_list("fa")  # only tose with fa in it
    reg.get_list()  # all recipes
    reg.find_candidate("rtools")
    reg.find_candidate("fa")  # several candidate


def test_remote_registry():
    reg = Registry("https://biomics.pasteur.fr/salsa/damona/registry.txt")
    reg.get_list()



from damona.config import Config
import shutil


def test_config():
    c = Config()
    c.read()


def test_config_creation():  # this is for the local cases

    c = Config(name="damona__testing__")
    if str(c.user_config_dir).endswith("damona__testing__"):
        shutil.rmtree(str(c.user_config_dir))



from damona.common import *

import os
import pytest

from . import test_dir


def test_no_path(monkeypatch):

    monkeypatch.delenv("DAMONA_PATH", raising=False)
    try:
        DamonaInit()
        assert False
    except:
        assert True
    try:
        Damona()
        assert False
    except:
        assert True


def test_path():
    # os.environ['DAMONA_PATH'] = '/tmp'
    d = Damona()
    d.config_path

    d.find_orphan_binaries()
    d.get_environments()
    d.find_orphan_images()


def test_ImageReader():

    try:
        ir = ImageReader(f"{test_dir}/data/testing_1.0.0")
        assert False
    except SystemExit:
        assert True

    ir = ImageReader(f"{test_dir}/data/testing_1.0.0.img")
    assert ir.version == "1.0.0"
    ir.md5
    ir.guessed_executable
    ir.is_orphan()
    ir.is_installed()
    print(ir)






import os
import builtins
import mock

from damona.environ import Environment
from damona.environ import Environ
from damona import Damona
from damona import script
import damona
from click.testing import CliRunner
import subprocess

def test_no_var(monkeypatch):
    monkeypatch.delenv("DAMONA_ENV", raising=False)
    try:
        Environ()
        assert False
    except:
        assert True


def test_environ(monkeypatch):
    manager = Damona()
    env = Environ()
    env.N
    env.environments
    env.create(".dummy_test")
    import mock

    with mock.patch.object(builtins, "input", lambda _: "y"):
        env.delete(".dummy_test")

    env.environment_names

    # this only prints the new PATH on the screen, so we need ti monkey patch the real env
    env.activate("base")
    monkeypatch.setenv("DAMONA_ENV", str(manager.damona_path / "envs" / "base"))
    assert env.get_current_env_name() == "base"
    env.deactivate()


def test_environment_no_var(monkeypatch):
    monkeypatch.delenv("DAMONA_PATH", raising=False)
    e = Environment("base")
    try:
        e.get_disk_usage()
        assert False
    except:
        assert True


def test_environment():
    e = Environment("base")
    e.get_installed_binaries()
    e.get_current_state()
    # cannot rename the base
    try:
        e.rename("base")
        assert False
    except SystemExit:
        assert True

    try:
        e = Environment("base_does_not_exist")
        assert False
    except:
        assert True

    env = Environ()
    env.create(".dummy_test")
    with mock.patch.object(builtins, "input", lambda _: "y"):
        this_env = Environment(".dummy_test")
        try:
            # cannot rename with existing  name
            this_env.rename(".dummy_test")
            assert False
        except SystemExit:
            assert True
        # cleanup
        env.delete(".dummy_test")



def test_create_bundle(tmpdir, monkeypatch):
    # make sure it exists
    runner = CliRunner()

    if "damona__testing__" not in Environ().environment_names:
        results = runner.invoke(script.env, ["--create", "damona__testing__"])

    manager = Damona()
    monkeypatch.setenv("DAMONA_ENV", str(manager.damona_path / "envs/damona__testing__"))
    cmd = "damona install fastqc --force"
    status = subprocess.call(cmd.split())


    directory = tmpdir.mkdir("bundle")
    destination = directory / "test"
    #print(destination)
    e = Environment("damona__testing__")
    e.create_bundle()
    os.remove("damona_damona__testing__.tar")

    # cleanup
    with mock.patch.object(builtins, "input", lambda _: "y"):
        results = runner.invoke(script.env, ["--delete", "damona__testing__"])










#
#  This file is part of Damona software
#
#  Copyright (c) 2020-2021 - Damona Development Team
#
#  File author(s):
#      Thomas Cokelaer <thomas.cokelaer@pasteur.fr>
#
#  Distributed under the terms of the 3-clause BSD license.
#  The full license is in the LICENSE file, distributed with this software.
#
#  website: https://github.com/cokelaer/damona
#  documentation: http://damona.readthedocs.io
#
##############################################################################
"""Provide some stats for admin"""
from damona import Registry
from damona.registry import Software
from damona.registry import ImageName
from damona import version

__all__ = ["stats"]


def stats():
    """Prints statistics about Damona

    ::

        from damona.admin import stats
        stats()

    """

    r = Registry()

    names = {ImageName(x.replace(":", "_") + ".img").name for x in r.get_list()}
    N = len(r.get_list())

    print(f"- version: {version}")
    print(f"- number of recipes:  {len(names)}")
    print(f"- number of versions: {N}")

    binaries = set()
    for x in r.get_list():
        s = Software(x.split(":")[0])
        binaries = binaries.union(set(s.binaries[s.releases.last_release]))
    N = len(binaries)
    print(f"- unique binaries: {N}")
